﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using CSVParser;
using QlmLicenseLib;
using System.Data;

namespace QlmImport
{


    // Usage: 
    //		CSVFileParser parser = new CSVFileParser();
    //		parser.Parse(@"pathto\myfile.csv");
    // (accepts either stream or filename)

    public class QlmOrderParser : CSVParser.CSVParser
    {
        QlmLicense license1 = null;
        
        public QlmOrderParser(QlmLicense license) : base()
        {
#if __Z__
			this.Hints.Add(new CSVHint(){ ColumnIndex=0, ColumnName = "Col1", DataType=CSVHintType.Int });
			this.Hints.Add(new CSVHint(){ ColumnIndex=1, ColumnName = "Col2", DataType=CSVHintType.String });
			this.Hints.Add(new CSVHint(){ ColumnIndex=2, ColumnName = "Col3", DataType=CSVHintType.QuotedString });
			this.Hints.Add(new CSVHint(){ ColumnIndex=3, ColumnName = "Col4", DataType=CSVHintType.String });
			this.Hints.Add(new CSVHint(){ ColumnIndex=4, ColumnName = "Col5", DataType=CSVHintType.QuotedString });
			this.Hints.Add(new CSVHint(){ ColumnIndex=5, ColumnName = "Col6", DataType=CSVHintType.QuotedString });
#endif

            license1 = license;

        }

        /// <summary>
        /// Parses the CurrentData property. The CurrentData property contains the parsed 
        /// comma-delimited line from the stream/file, and this method is called after each line is 
        /// parsed.
        /// </summary>
        protected override void ProcessFields(bool isMalformed)
        {
            if (this.CurrentData != null && !isMalformed)
            {
                string orderID = string.Empty;

                // TO-DO: Your stuff
                try
                {
                    orderID = this.FindValue("orderID", "");

                    int productID = this.FindValue("productID", 0);
                    int major = this.FindValue("major", 0);
                    int minor = this.FindValue("minor", 0);

                    string customerEmail = this.FindValue("email", "");
                    int numLicenses = this.FindValue("numLicenses", 1);
                    string userData = this.FindValue("userData", "");
                    string affiliateID = this.FindValue("affiliateID", "");

                    DateTime expiryDate = this.FindValue("expiryDate", DateTime.MinValue);
                    int expiryDuration = this.FindValue("expiryDuration", -1);

                    DateTime maintenancePlanExpiryDate = this.FindValue("maintenancePlanExpiryDate", DateTime.MinValue);

                    string response = string.Empty;

                    license1.DefineProduct(productID, "", major, minor, "", "");

                    if (OrderExists(orderID) == false)
                    {

                        license1.CreateOrderEx(string.Empty, customerEmail, null, numLicenses, true, "5.0.00", string.Empty, userData, affiliateID,
                                                    expiryDate, expiryDuration, orderID, 0, out response);

                        ILicenseInfo li = new LicenseInfo();
                        string message = string.Empty;

                        if (license1.ParseResults(response, ref li, ref message))
                        {
                            // all good
                            ImportCount++;

                            if (maintenancePlanExpiryDate != DateTime.MinValue)
                            {
                                if (license1.SetMaintenancePlanRenewalDate(string.Empty, li.ActivationKey, maintenancePlanExpiryDate, out message) == false)
                                {
                                    SbResult.AppendFormat("Error setting maintenance plan on order {0}. Error: {1}\r\n", orderID, message);
                                }
                            }

                            // Publish your existing license key to the server
                            // Update the field name LicenseKey to the field that corresponds to your existing license key
                            string legacyKey = this.FindValue("licenseKey", "");
                            if (!String.IsNullOrEmpty(legacyKey))
                            {
                                string errorMessage;
                                SetLegacyKey(license1, string.Empty, li.ActivationKey, legacyKey, out errorMessage);
                            }
                        }
                        else
                        {
                            // error adding the user
                            SbResult.AppendFormat("Error adding order {0}. Error: {1}\r\n", orderID, message);
                        }
                    }
                }
                catch (FindValueException fvex)
                {
                    // this just gets rid of the compiler "declared but never used" warning.
                    if (fvex != null) { }

                    //TO-DO: react to an exception thrown because the value found could not be cast 
                    // to the expected type.
                    SbResult.AppendFormat("Error adding order {0}. Error: {1}\r\n", orderID, fvex.Message);
                }
            }
            else
            {
                SbResult.Append("The data is malformed.\r\n");
            }
        }

        private bool SetLegacyKey (QlmLicense license, string webServiceUrl, string activationKey, string legacyKey, out string errorMessage)
        {
            errorMessage = string.Empty;

            // You can specify multiple properties in the licenseArguments tag. Here we're only setting one attribute.

            // Note the UserData1 attribute value is prefixed with  \" followed by a single quote ' and then suffixed with a single quote ' followed by \".

            string licenseUpdateData = String.Format("<licenseArguments LegacyKey=\"'{0}'\" />", legacyKey);

            // Call the web service
            string response;
            license.UpdateLicenseInfo(webServiceUrl, activationKey, licenseUpdateData, out response);

            ILicenseInfo li = new LicenseInfo();
            return license.ParseResults(response, ref li, ref errorMessage);
        }

        protected override void Finished()
        {
            //TO-DO:	Celebration that could include dancing naked around a fire, singing songs 
            //			of a successful parsing event.

            // At this point you can examine the properties that show valid, invalid, and corrected 
            // line counts, as well as total lines processed (excluding the header row and blank 
            // lines).

        }

        private bool OrderExists (string orderID)
        {
            bool ret = false;
            string response = string.Empty;

            if (String.IsNullOrEmpty (orderID))
            {
                return false;
            }

            DataRowCollection drc = license1.GetOrder(string.Empty, orderID, out response);

            ret = (drc == null) ? false : true;

            return ret;
        }

    }
}

